     /********************************************************************
      ***                      Gestion SD Card                         ***
      ***               Fonctions des couches basses                   ***
      ***                      dsPIC30F4011                            ***
      ***                                                              ***
      ***  Fichier "source" : SD_Card.c                                ***
      ***  Auteur : CREMMEL Marcel                                     ***
      ***  Date   : 24/09/2007                                         ***
      ********************************************************************/

#include <p30f4011.h>
#include "Hardware.h"
#include "SD_Card.h"
#include "Timer3.h"   // Pour Delay() et InitTMR3()
#include <stdlib.h>

/*------------------------------------------------------------------------------
                                  Variables                              
------------------------------------------------------------------------------*/

// Indicateurs de fonctionnement
extern struct 
{
 unsigned SDC_CD_OK : 1; // Dtection prsence carte (avec anti-rebonds)
 unsigned SDC_CD_old: 1; // Etat prcdent du contact CD
 unsigned SDC_CD_New: 1; // Nouvel tat stable du contact CD
 unsigned isSDMMC   : 1; // Indicateur de carte SDMMC 
 unsigned isWP      : 1; // Indicateur de protection en criture
 unsigned unused	: 11;
}FlagsSD;
// Couche physique
int Cpt_AR_SDC_CD;   // Compteur anti-rebonds dtection carte
// Dclaration multitypes du buffer pour un secteur
extern Secteur *msd_buffer; // Buffer de lecture et d'criture d'un secteur
// Registre CSD
extern CSD gblCSDReg;       // Registre CSD : Card Specific Data (128 bits ou 16 octets)

/*------------------------------------------------------------------------------
                              Constantes mmorises                      
------------------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 Table "sdmmc_cmdtable"
  - Informations sur chaque commande "sdmmc" :
     - code de la commande
     - valeur de l'ventuel CRC
     - nature de la rponse (R1, R1b, R2 ou R3)
     - si la commande est suivie de donnes
 Notes: la fonction CRC est inhibe ds que possible ou viter son calcul
        (valeur 0xFF dans ce cas) 
-----------------------------------------------------------------------*/
const typSDC_CMD sdmmc_cmdtable[] =
{
 //     cmd               crc   response  Donnes
 {cmdGO_IDLE_STATE,       0x95,   R1,      NODATA},   // CMD0 : Reset to Idle State
 {cmdSEND_OP_COND,        0xF9,   R1,      NODATA},   // CMD1 : Dclenche le processus d'init.
 {cmdSEND_CSD,            0xAF,   R1,      MOREDATA}, // CMD9 : Lecture du registre CSD
 {cmdSEND_CID,            0x1B,   R1,      MOREDATA}, // CMD10: Lecture du registre CID
 {cmdSTOP_TRANSMISSION,   0xC3,   R1,      NODATA},   // CMD12: Termine une op. R/W blk mult
 {cmdSEND_STATUS,         0xAF,   R2,      NODATA},   // CMD13: Lecture du registre de status
 {cmdSET_BLOCKLEN,        0xFF,   R1,      NODATA},   // CMD16: Affecte la longueur des blocs
 {cmdREAD_SINGLE_BLOCK,   0xFF,   R1,      MOREDATA}, // CMD17: Lecture d'un bloc
 {cmdREAD_MULTI_BLOCK,    0xFF,   R1,      MOREDATA}, // CMD18: Lecture continuelle de blocs
 {cmdWRITE_SINGLE_BLOCK,  0xFF,   R1,      MOREDATA}, // CMD24: Ecriture d'un bloc
 {cmdWRITE_MULTI_BLOCK,   0xFF,   R1,      MOREDATA}, // CMD25: Ecriture continuelle de blocs
 {cmdTAG_SECTOR_START,    0xFF,   R1,      NODATA},   // CMD32: Adresse dbut  effacer
 {cmdTAG_SECTOR_END,      0xFF,   R1,      NODATA},   // CMD33: Adresse fin  effacer
 {cmdUNTAG_SECTOR,        0xFF,   R1,      NODATA},   // CMD34
 {cmdTAG_ERASE_GRP_START, 0xFF,   R1,      NODATA},   // CMD35
 {cmdTAG_ERASE_GRP_END,   0xFF,   R1,      NODATA},   // CMD36
 {cmdUNTAG_ERASE_GRP,     0xFF,   R1,      NODATA},   // CMD37
 {cmdERASE,               0xDF,   R1b,     NODATA},   // CMD38: Effacer les blocs slectionns
 {cmdLOCK_UNLOCK,         0x89,   R1b,     NODATA},   // CMD49
 {cmdSD_APP_OP_COND,      0xE5,   R1,      NODATA},   // ACMD41: Dclenche l'initialisation
 {cmdAPP_CMD,             0x73,   R1,      NODATA},   // CMD55: La commande suivante est de type
                                                      //        "Application specific"
 {cmdREAD_OCR,            0x25,   R3,      NODATA},   // CMD58: Lecture registre OCR
 {cmdCRC_ON_OFF,          0x25,   R1,      NODATA}    // CMD59: CRC ON ou OFF
};

/*------------------------------------------------------------------------------
             Dclarations des fonctions en mmoire flash
------------------------------------------------------------------------------*/

/*--------------------------------------------------------------------
  Function    : void SDC_CD_AR(void)
  Description : Lecture de l'tat actuel du contact CD de la SD Card
                avec fonction "anti-rebonds".
  Traitement : - lance par le programme d'interruption "_T1Interrupt" (10Hz)
               - lecture tat actuel
               - si tat stable pdt 1 seconde -> nouvel tat stable dtect
                 et plac ds SDC_CD_OK
               - SDC_CD_New positionn si chgt d'tat stable
----------------------------------------------------------------------*/
void SDC_CD_AR(void)
{
 unsigned int SDC_CD_actuel;
 SDC_CD_actuel=!SDC_CD; // 1 : prsence, 0 : absence de carte 
 // Dtection d'un changement d'tat
 if (SDC_CD_actuel!=FlagsSD.SDC_CD_old) 
  // Etat diffrent entre 2 lectures
  Cpt_AR_SDC_CD=10; // R-initialisation du compteur anti-rebonds (10 secondes)
 else
 {// Mme tat entre 2 lectures
  if (Cpt_AR_SDC_CD!=0)
  {// Dtection rebonds en cours
   Cpt_AR_SDC_CD--; // Dcomptage "anti-rebonds"
   if (Cpt_AR_SDC_CD==0)
   {// Mme tat pdt 10 lectures = 1s
    if (FlagsSD.SDC_CD_OK!=SDC_CD_actuel) // Chgt d'tat stable ?
    {
     FlagsSD.SDC_CD_OK=SDC_CD_actuel; // Nouvel tat stable
     FlagsSD.SDC_CD_New=1;            // Chgt d'tat stable
    }
   }
  }
 }
 FlagsSD.SDC_CD_old=SDC_CD_actuel;
}

/*--------------------------------------------------------          
  Function   : byte IsWriteProtected()
  Description: lecture du switch de protection en criture
  Paramtres : Aucun
  Retour     : TRUE si protg en criture
               FLASE dans le cas contraire
-----------------------------------------------------------*/
byte IsWriteProtected(void)
{
 if (SDC_WP) return TRUE;
 else return FALSE;
}

/*--------------------------------------------------------          
  Function:    void InitSPI(int Div)
  Description: Initialisation coupleur SPI pour cartes SD
    - Master 8 bits sans gestion de trame
    - CKE=1, CKP=0 et SMP=0
    - SS non utili
  Paramtre : Div = 0 : prdivision Fcy par 64
                  = 1 : prdivision Fcy par 16
                  = 2 : prdivision Fcy par 4
                  = 3 : prdivision Fcy par 1
-----------------------------------------------------------*/
void InitSPI(int Div)
{
 SPI1CON=0x013C|Div; // FRMEN=SPIFSD=0 : pas de gestion des trames
                     // DISSD0=0       : SDO1 utilis
                     // MOD16=0        : mode 8 bits
                     // SMP=0          : SDI1 chantillonn au milieu
                     // CKE=1          : SDO1 change qd SCK1 passe de l'tat 
                     //                  actif  l'tat de repos ("Idle")
                     // SSEN=0         : SS non utilis
                     // CKP=0          : tat de repos SCK1=0
                     // MSTEN=1        : mode Master
                     // SPRE<2:0>=0    : prdivision secondaire=1
                     // PPRE<1:0>=Div  : voir ci-dessus
 SPI1STAT=0x8000;    // SPIEN=1        : coupleur SDI actif
                     // SPISIDL=0      : module oprationnel en mode Idle
                     // SPIROV,SPITBF,SPIRBF : indicateurs
}

/*------------------------------------                           
  Function   : void CloseSPI(void)
  Description: Arrt du coupleur SPI
--------------------------------------*/
void CloseSPI()
{  
  _SPI1IE = 0;            /* Disable the Interrupt bit in the 
                             Interrupt Enable Control Register */
  SPI1STATbits.SPIEN = 0; /* Disable the module. All pins controlled
                             by PORT Functions */
  _SPI1IF = 0;            /* Disable the Interrupt flag bit in the
                             Interrupt Flag Control Register */    
}

/*---------------------------------------------------------         
  Function:    unsigned char WriteSPI(unsigned char data)
  Description: Transmission et rception simultane d'un octet 
               sur le coupleur SPI
  Paramtre : data = octet  transmettre sur SDO
  Retour    : octet lu sur SDI
------------------------------------------------------------*/
unsigned char WriteSPI(unsigned char data)
{
 SPI1BUF=data; // Affectation du buffer SPI : dbut d'une transmission/rception
 // Attendre la rception d'un nouveau caractre, donc la fin de transmission
 while (!SPI1STATbits.SPIRBF) {};
 //SPI1STATbits.SPIROV=0;
 return SPI1BUF;  // Lecture octet reu et Raz SPIRBF
}
// Macros associes pour rendre le source plus lisible
#define ReadSPI()  WriteSPI(0xFF) // Transmission octet neutre pour rception
#define ClockSPI() WriteSPI(0xFF) // Pour produire 8 impulsions d'horloge

/*-------------------------------------------------------------
  Function   : void SocketInitialize(void)
  Description: Initialisation des ports utiliss par la SD Card
  Paramtres : Aucun
  Retour     : Aucun
---------------------------------------------------------------*/
void SocketInitialize(void)
{    
 // Couper l'alimentation de la carte mmoire
 SDC_POWER=1;         // Connecteur SD Card non aliment au reset
 SDC_POWER_DIR=OUTPUT;// Port SDC_POWER en sortie
 SDC_CD_DIR=INPUT;    // Port "Card Detect" en entre
 SDC_CS=1;            // CS de la SD Card  "1"
 SDC_CS_DIR=INPUT;    // Port SDC_CS en entre si SD Card non alimente
 SDC_WP_DIR=INPUT;    // Port "Write Protect" en entre
 #ifdef STATUSLED
  STRTRIS=OUTPUT;
  STWTRIS=OUTPUT;
  STRLED=0; // Led indicatrice "lectures" teinte
  STWLED=0; // Led indicatrice "critures" teinte
 #endif
}// end SocketInitialize

/*-------------------------------------------------
  Function   : void ShutdownMedia(void)
  Description: Coupure alimentation de la carte SD
  Paramtres : Aucun
  Retour     : Aucun
---------------------------------------------------*/
void ShutdownMedia(void)
{
 SDC_CS=1;         // D-slection de la carte                              
 SDC_CS_DIR=INPUT; // Port SDC_CS en entre si SD Card non alimente
 CloseSPI();       // Ports SPI en HiZ
 SDC_POWER=1;      // Coupure alimentation        
 free(msd_buffer); // Librer la mmoire alloue
}

/*--------------------------------------------------------------------------------
  Function   : SDC_RESPONSE SendSDCCmd(BYTE cmd, DWORD address)
  Description: 
    - Prpation de la trame de la commande
    - Rponse de la carte (R1 ou R2) dans la valeur retourne
  Paramtres : 
    - cmd : code de la commande (type "sdmmc_cmd")
    - adress : argument de la commande
  Retour     : rponse de la carte SD (un octet)
    - 0x00 ou 0x01 : commande bien reue, sinon erreur : 
      - Bit 0 IN_IDLE_STATE:1; // La carte est en  mode "Idle" 
                               // et excute la phase d'initialisation
      - Bit 1 ERASE_RESET  :1; // Acquitement de l'excution d'une commande 
                               // d'arrt d'effacement
      - Bit 2 ILLEGAL_CMD  :1; // Commande illgale
      - Bit 3 CRC_ERR      :1; // Le CRC de la dernire commande est incorrect
      - Bit 4 ERASE_SEQ_ERR:1; // Erreur ds la squence d'effacement commande
      - Bit 5 ADDRESS_ERR  :1; // Erreur d'adresse 
      - Bit 6 PARAM_ERR    :1; // Erreur dans les arguments de la dernire commande
      - Bit 7 B7           :1; // Toujours  0
  Attention : le signal SDC_CS n'est pas mis  "1"  la fin de cette fonction.
              s'il reste des donnes  transmettre (criture ou lecture)
----------------------------------------------------------------------------------*/
SDC_RESPONSE SendSDCCmd(byte cmd, dword address)
{        
 word timeout=8; // Dlai max entre cmde et rponse = NCR = 8 octets
 byte index;
 SDC_RESPONSE response; // Pour mmoriser localement la rponse
 CMD_PACKET CmdPacket;  // Structure de la commande
	    
 SDC_CS=0; // Card Select  
 // Prpation de la commande complte
 CmdPacket.cmd    =sdmmc_cmdtable[cmd].CmdCode; // Code commande
 CmdPacket.address=address;                     // Argument (4 octets) 
 CmdPacket.crc    =sdmmc_cmdtable[cmd].CRC;     // CRC    
 // Transmission de la trame   
 WriteSPI(CmdPacket.cmd);   // Send Command
 WriteSPI(CmdPacket.addr3); // Argument MSB
 WriteSPI(CmdPacket.addr2);
 WriteSPI(CmdPacket.addr1);
 WriteSPI(CmdPacket.addr0); // Argument LSB
 WriteSPI(CmdPacket.crc);   // Send CRC
 // La commande est-elle suivie d'une rponse ?
 if(sdmmc_cmdtable[cmd].responsetype==R1||sdmmc_cmdtable[cmd].responsetype==R1b)
 {// de type R1 (un seul octet) ou R1b (1 octet):
  do
  {// Lecture d'octets jusqu' octet diffrent de 0xFF ou "Timeout"
   response.r1._byte=ReadSPI();
   timeout--; // Le dlai max de la rponse est NCR=4x8=32 priodes CLK
  } while ((response.r1._byte==0xFF)&&(timeout!=0));
 }
 else if(sdmmc_cmdtable[cmd].responsetype==R2)
 {// de type R2 (commande SEND_STATUS):
  ReadSPI(); // Dlai = 1 octet
  response.r2._byte1=ReadSPI(); // Rception des 2 octets "R2"
  response.r2._byte0=ReadSPI();
 }    
 if(sdmmc_cmdtable[cmd].responsetype==R1b)
 {// de type R1b : attendre la fin de l'opration en cours (signal "busy")
  response.r1._byte=0x00;
  for(index=0; (index<0xFF)&&(response.r1._byte==0x00); index++)
  {
   timeout = 0xFFFF; // Timeout = lecture de 65536 octets !	           
   do
   {// Lectures d'octets de la carte jusqu' ce qu'elle soit "ready"
    // (octet lu = 0) ou "timeout"
    response.r1._byte=ReadSPI(); // Lecture d'un octet
    timeout--;
   } while ((response.r1._byte==0x00)&&(timeout!=0)); 
  }
 }       
 WriteSPI(0xFF); // Impos par la norme pour terminer l'opration (5.1.8)
 // Reste-t-il des donnes  transmettre ?
 //   - non : SDC_CS revient  "1"
 //   - oui : SDC_CS est maintenu  "0"
 if(!(sdmmc_cmdtable[cmd].moredataexpected)) 
  {
   Delay(Delai_48uS); // 48S
   SDC_CS=1; // D-slection SD Card
   Delay(Delai_mini); // 16S
  } 
 return(response);        
}

/*-------------------------------------------------------------------          
  Function   : SDC_Error CSDRead(void)
  Description: Lecture du registre CSD (Card Specific Data) :
    - transmission de la commande CMD9 SEND_CSD
    - rception des 128 bits du registre (16 octets). Infos extraites :
        - capacit de la carte
        - longueur (gblBlkLen) et nombre de blocs (gblNumBlks)
  Paramtres : buffer : destination des donnes lues 
  Retour     : nature de l'ventuelle erreur (voir SDC_Error)
---------------------------------------------------------------------*/
SDC_Error CSDRead(void)
{
 int index;
 word timeout=0x2ff;
 SDC_RESPONSE response;
 byte data_token;
 SDC_Error status = sdcValid;
 byte cmd=SEND_CSD;
 dword address=0x00;
 CMD_PACKET CmdPacket;
	    
 SDC_CS=0; //Card Select (CS="0")   
 // Prparation de la commande SEND_CSD (CMD9)
 CmdPacket.cmd    = sdmmc_cmdtable[cmd].CmdCode; // Code
 CmdPacket.address= address;                     // Argument
 CmdPacket.crc    = sdmmc_cmdtable[cmd].CRC;     // CRC
 WriteSPI(CmdPacket.cmd);   // Transmission Code commande
 WriteSPI(CmdPacket.addr3); // Argument MSB
 WriteSPI(CmdPacket.addr2);
 WriteSPI(CmdPacket.addr1);
 WriteSPI(CmdPacket.addr0); // Argument LSB
 WriteSPI(CmdPacket.crc);   // Transmission CRC en dernier
 // Attente et rception de la rponse de type R1
 //timeout=8; // *** A tester *** Dlai max entre cmde et rponse = NCR = 8 octets
 do
 {
  response.r1._byte=ReadSPI();
  timeout--;
 } while ((response.r1._byte==0xFF) && (timeout!=0));
 // S'assurer que la carte est prte
 if (response.r1._byte != 0x00) status=sdcCardBadCmd;
 else
 {
  index = 0x2FF;                                      
  // Attendre le jeton de dpart et le bloc de donnes   
  do
  {// Attendre le premier octet diffrent de 0xFF
   data_token=ReadSPI();
   index--;
  } while ((data_token==SDC_FLOATING_BUS) && (index!=0));    
  // Le premier octet doit tre le jeton DATA_START_TOKEN
  if ((index == 0)||(data_token!=DATA_START_TOKEN))
     status = sdcCardTimeout;
  else
  {// Oui : rception des 16 octets suivants (128 bits du registre CSD
   for(index=CSD_SIZE-1; index>=0; index--) gblCSDReg._byte[index]=ReadSPI();  
  }
 // Le transfert se termine avec les 16 bits du CRC ( **** TEST ****)
 WriteSPI(0xFF);WriteSPI(0xFF); // Lecture simple, sans prise en compte
 WriteSPI(0xFF); // Impos par la norme pour terminer l'opration (5.1.8)
 }
 Delay(5*Delai_mini); // 5*16S
 SDC_CS=1;          // D-selection carte SD (CS="1")
 Delay(Delai_mini); // 16S
 return(status);
}

/*---------------------------------------------------------------------------
  Function   : SDC_Error SectorRead(uint32 sector_addr, Secteur *buffer)
  Description: la fonction lit 512 octets  partir de l'adresse "sector_addr"
               et range les octets dans "buffer"
  Paramtres :
   - sector_addr : adresse du secteur; chaque secteur comporte un bloc de 512 
                   octets
   - buffer : destination des donnes lues  (buffer de 512 octets)
  Retour     : voir SDC_Error
-----------------------------------------------------------------------------*/
SDC_Error SectorRead(uint32 sector_addr, Secteur *buffer)
{
 uint index;
 SDC_RESPONSE response; // Variable pour la rponse de la commande
 byte data_token;       // Variable pour le jeton
 SDC_Error status = sdcValid; // Pas d'erreur  priori
 #ifdef STATUSLED
  STRLED=1; // Allumer la Led indicatrice de lecture
 #endif
 // Envoi de la commande de lecture d'un bloc de 512 octets
 // Le dcalage de 9 bits correspond  la taille des secteurs (2^9=512)
 response=SendSDCCmd(READ_SINGLE_BLOCK,(sector_addr<<9));
 // Vrifier que la commande a bien t accepte
 if(response.r1._byte!=0x00) status=sdcCardBadCmd;
 else
 {// Octet de rponse = 0x00 : commande accepte -> lecture du bloc
  index=0x2FF; // Variable "index" utilise ici comme compteur de "Timeout"
  // Attente du jeton "Start Block" et du bloc de donnes
  do
  {// Le premier octet reu diffrent de 0xFF est le jeton
   data_token=ReadSPI();
   index--; // Dcompteur Timeout
  } while ((data_token==SDC_FLOATING_BUS) && (index!=0));
  // Ce jeton doit tre DATA_START_TOKEN
  if ((index==0)||(data_token!=DATA_START_TOKEN)) status=sdcCardTimeout;
  else
  {// Le jeton est correct : les 512 octets de donnes suivent
   for (index=0; index<SDC_SECTOR_SIZE; index++) buffer->octets[index]=ReadSPI();
   // Le transfert se termine avec les 16 bits du CRC
   ReadSPI(); ReadSPI(); // Lecture simple des 2 octets, sans prise en compte
   //status = mmcCardCRCError;
  }
  WriteSPI(0xFF); // Le 5.1.8 du document "ProdManualSDCardv1.9.pdf" impose 
                  // la production de 8 impulsions d'horloge  la fin des changes 
                  // avant un arrt prolong de l'horloge.
 }
 // Dlai supplmentaire pour s'assurer de l'arrt de l'horloge
 Delay(3*Delai_mini); // 3*16S
 SDC_CS=1;            // D-selection de la carte SD
 Delay(Delai_mini);   // 16S
 #ifdef STATUSLED
  STRLED = 0;         // Eteindre la Led indicatrice de lecture
 #endif 
 return status;
}

/*---------------------------------------------------------------------------------          
  Function   : SDC_Error SectorWrite(DWORD sector_addr, Secteur *buffer)
  Description: la fonction crit les 512 octets de "buffer"  partir de 
               l'adresse "sector_addr"
  Paramtres :
   - sector_addr : adresse du secteur; chaque secteur comporte un bloc de 512 octets
   - buffer : pointeur vers la source des donnes  crire (tableau de 512 octets)
  Retour     : voir SDC_Error
----------------------------------------------------------------------------------*/
SDC_Error SectorWrite(uint32 sector_addr, Secteur *buffer)
{
 word         index;
 byte         data_response;       
 SDC_RESPONSE response;      // Variable pour la rponse de la commande
 SDC_Error    status=sdcValid; 	
 #ifdef STATUSLED
  STWLED=1; // Allumer la Led indicatrice d'criture
 #endif	
 // Envoi de la commande d'criture d'un bloc de 512 octets
 // Le dcalage de 9 bits correspond  la taille des blocs (2^9=512)
 response=SendSDCCmd(WRITE_SINGLE_BLOCK,(sector_addr << 9));
 // Vrifier que la commande a bien t accepte
 if(response.r1._byte!=0x00) status=sdcCardBadCmd;    
 else
 {// Octet de rponse = 0x00 : commande accepte -> criture du bloc
  WriteSPI(DATA_START_TOKEN); // On commence par transmettre le jeton START
  // Puis les 512 octets
  for(index=0;index<SDC_SECTOR_SIZE;index++) WriteSPI(buffer->octets[index]);
  // Puis transmission des 16 bits CRC:
  WriteSPI(0xFF); WriteSPI(0xFF); // Non calcul
  // Lecture immdiate de l'octet de rponse
  data_response=ReadSPI();
  if ((data_response & 0x0F)!=DATA_ACCEPTED) status=sdcCardDataRejected;
  else
  {// Rponse OK : attente fin de l'criture effective en mmoire flash
   index=0; // Timeout
   do
   {// Lectures successives jusqu' rception d'un octet de rponse non nul ou Timeout
    data_response=ReadSPI();
    index++;
   } while ((data_response==0x00)&&(index!=0));
   // Test Timeout (65536x8 priodes d'horloge !)         
   if (index==0) status=sdcCardTimeout;
  }
  WriteSPI(0xFF); // Impos par la norme pour terminer l'opration (5.1.8)
 }
 Delay(3*Delai_mini); // 3*16S pour attendre la transmission du dernier octet
 SDC_CS=1;            // D-selection de la carte SD
 Delay(Delai_mini);   // 16S
 #ifdef STATUSLED
  STWLED=0; // Eteindre la Led indicatrice d'criture
 #endif
 return(status);
}

/*---------------------------------------------------------------------------------
  Function   : SDC_Error MediaInitialize()
  Description: Initialisation de la carte SD et des variables associes
  Paramtre  : indicateurs sur la carte :
    isSDMMC : type MMC
    isWP    : protge en criture
  Retour     : nature de l'erreur
    sdcValid                Indicateur de carte valide
    sdcCardInitCommFailure  Aucune communication n'a pu s'tablir avec la carte
    sdcCardNotInitFailure   La carte n'est pas entre dans la phase "initialisation"
    sdcCardInitTimeout      "Timeout" pdt l'initialisation
    sdcCardTypeInvalid      Le type de carte n'a pas pu tre tabli
    sdcCardBadCmd           La carte n'a pas reconnu la commande
    sdcCardTimeout          "Timeout" pdt une lecture, criture ou effacement   
    sdcCardCRCError         Erreur CRC pdt une lecture 
    sdcCardDataRejected     Le "CRC" des donnes ne correspond pas
    sdcEraseTimedOut        "Timeout" pdt effacement
  Note : SDC_CS reste  "0" au retour en l'absence d'erreur
------------------------------------------------------------------------------------*/
SDC_Error MediaInitialize(void)
{
 uint16       timeout;
 SDC_Error    status=sdcValid, CSDstatus=sdcValid;
 SDC_RESPONSE response; 
 // Raz indicateurs  priori
 FlagsSD.isSDMMC = FlagsSD.isWP = 0;
 SDC_CS=1; // D-selection carte SD
 // Mise sous tension de la carte SD
 SDC_POWER=0;
 // Attendre la stabilisation de la tension d'alimentation
 Delay(Delai_40ms);
 SDC_CS_DIR=OUTPUT; // Port CS_SD en sortie
 // A la mise sous tension, la carte SD est en mode "drain ouvert" et n'accepte
 // pas des horloges suprieures  400kHz
 InitSPI(0); // Coupleur SPI en mode "master" 8 bits; horloge = Fcy/64 = 250kHz
 // La carte SD ncessite 80 priodes d'horloge pour tre oprationnelle
 for(timeout=0; timeout<10; timeout++) WriteSPI(0xFF);// 10 octets x 8 bits
 Delay(Delai_500us); // Attendre arrt horloge (transmission du dernier octet)
 SDC_CS=0;   // Card Select
 Delay(Delai_500us);
 // Commande CMD0=GO_IDLE_STATE : reset "software" de la carte
 response=SendSDCCmd(GO_IDLE_STATE,0x0);// Transm. et rception de la rponse "R1"
 if(response.r1._byte==SDC_BAD_RESPONSE)
 {
  status=sdcCardInitCommFailure; // La carte est non connecte ou a une dfaillance
  goto InitError; // Fin prmature
 }
 // La carte est-elle prte (ready) ?
 if (response.r1._byte != 0x01) //0x01 = No Err & Busy Initializing
 {
  status=sdcCardNotInitFailure; // La carte n'est pas prte
  goto InitError; // Fin prmature
 }
 // En mode "Idle", la cmde CMD1 doit tre rpte jusqu' ce que la carte soit prte
 timeout=0xFFF; // Timeout = 4096x8 priodes d'horloge !
 do
 {
  response=SendSDCCmd(SEND_OP_COND,0x0); // Commande CMD1 (initialisation)
  timeout--; // Le processus d'initialisation de la carte prend de 50mS  500mS
 } while (response.r1._byte!=0x00 && timeout!=0);       
 // Ragir si la carte n'est pas initialise
 if(timeout==0)
 {
  status=sdcCardInitTimeout; // Initialisation dfaillante aprs Timeout !
  goto InitError; // Fin prmature
 }                        
 else  
 {// La carte SD est prte : 
  CSDstatus=CSDRead(); // Lecture du registre CSD de la carte
  if(!CSDstatus)       // Aucune anomalie ?
   {//Oui : on peut monter l'horloge
    InitSPI(1);        // Mode "master" 8 bits pour cartes SD; horloge = Fcy/16 = 1MHz
   }
  else 
   {//Non : la carte est dfaillante
    status=sdcCardTypeInvalid;
    goto InitError; // Fin prmature
   } 
 }
 // Arrt CRC7 si possible; commande invalide pour certaines cartes (CMD59)
 response=SendSDCCmd(CRC_ON_OFF,0x0);
 // Affectation de la longueur des blocs  512
 SendSDCCmd(SET_BLOCKLEN,BLOCKLEN_512);
 // Affectation de l'indicateur de protection d'criture
 if (IsWriteProtected()) FlagsSD.isWP=TRUE;
 // Lecture rpte d'un secteur jusqu' l'absence d'erreur. Certaines cartes "San"
 // n'acceptent pas cette fonction d'initialisation pour diffrentes raisons
 for (timeout=0xFF;
	  timeout>0&&SectorRead(0x0,msd_buffer)!=sdcValid;
	  timeout--) {}
 // Lecture du secteur dfaillante ?
 if(timeout==0)  
 {// Oui :      
  status = sdcCardNotInitFailure; // La carte est dfaillante
  goto InitError; // Fin prmature
 }
 return(status); // Retour en l'absence d'erreur
    
InitError:
 SDC_CS=1; // D-slection de la carte 
 return(status);                            
}

