     /********************************************************************
      ***                      Gestion SD Card                         ***
      ***               Fonctions des couches basses                   ***
      ***                      dsPIC30F4011                            ***
      ***                                                              ***
      ***  Fichier "header" : SD_Card.h                                ***
      ***  Auteur : CREMMEL Marcel                                     ***
      ***  Date : 24/09/2007                                           ***
      ********************************************************************/

#ifndef SD_Card_Defs
#define SD_Card_Defs

#include "Typedefs_SD.h" // SDC_Error, CSD, CID, SDCSTATE, sdmmc_cmd, RESP, CMD_PACKET
                         // REPONSE_1, REPONSE_2, SDC_REPONSE, typSDC_CMD
#include "Typedefs_FAT.h"// Secteur

/*------------------------------------------------------------------------------
                      Dclaration des quivalences
------------------------------------------------------------------------------*/

// Divers
#define FALSE	0
#define TRUE	!FALSE
#define INPUT	1
#define OUTPUT	0

// SD-Card
#define MOREDATA        !0
#define NODATA          0
#define SDC_SECTOR_SIZE 512
#define CSD_SIZE        16   // 16 octets du registre CSD

/* Command Operands */
#define BLOCKLEN_64                 0x0040
#define BLOCKLEN_128                0x0080
#define BLOCKLEN_256                0x0100
#define BLOCKLEN_512                0x0200

#define SDC_FLOATING_BUS    		0xFF
#define SDC_BAD_RESPONSE    		SDC_FLOATING_BUS
#define SDC_ILLEGAL_CMD     		0x04
#define SDC_GOOD_CMD        		0x00

/* Data Token */
#define DATA_START_TOKEN            0xFE
#define DATA_MULT_WRT_START_TOK     0xFC
#define DATA_MULT_WRT_STOP_TOK      0xFD

/* Data Response */
#define DATA_ACCEPTED               0b00000101
#define DATA_CRC_ERR                0b00001011
#define DATA_WRT_ERR                0b00001101

// Commandes "SD Card" (SPI Bus Mode)
// Bit 7 = 0, Bit 6 = 1, Bits 5-0=Command Code
#define     cmdGO_IDLE_STATE        0x40		//0 : code de la cmd sur 6 bits LSB
#define     cmdSEND_OP_COND         0x41		//1        
#define     cmdSEND_CSD             0x49		//9
#define     cmdSEND_CID             0x4a		//10
#define     cmdSTOP_TRANSMISSION    0x4c		//12
#define     cmdSEND_STATUS          0x4d		//13
#define     cmdSET_BLOCKLEN         0x50		//16
#define     cmdREAD_SINGLE_BLOCK    0x51		//17
#define     cmdREAD_MULTI_BLOCK     0x52		//18
#define     cmdWRITE_SINGLE_BLOCK   0x58		//24    
#define     cmdWRITE_MULTI_BLOCK    0x59		//25
#define     cmdTAG_SECTOR_START     0x60		//32
#define     cmdTAG_SECTOR_END       0x61		//33
#define     cmdUNTAG_SECTOR         0x62		//34
#define     cmdTAG_ERASE_GRP_START  0x63		//35 
#define     cmdTAG_ERASE_GRP_END    0x64		//36
#define     cmdUNTAG_ERASE_GRP      0x65		//37
#define     cmdERASE                0x66		//38
#define     cmdSD_APP_OP_COND       0x69		//41
#define     cmdLOCK_UNLOCK          0x71		//49
#define     cmdAPP_CMD              0x77		//55
#define     cmdREAD_OCR             0x7a		//58
#define     cmdCRC_ON_OFF           0x7b		//59

/*------------------------------------------------------------------------------
           Dclarations des prototypes des fonctions en mmoire flash
------------------------------------------------------------------------------*/

/*--------------------------------------------------------------------
  Function    : void SDC_CD_AR(void)
  Description : Lecture de l'tat actuel du contact CD de la SD Card
                avec fonction "anti-rebonds".
  Traitement : - lance par le programme d'interruption "_T1Interrupt" (10Hz)
               - lecture tat actuel
               - si tat stable pdt 1 seconde -> nouvel tat stable dtect
                 et plac ds SDC_CD_OK
               - SDC_CD_New positionn si chgt d'tat stable
-----------------------------------------------------------------------*/
void SDC_CD_AR(void);

/*--------------------------------------------------------          
  Function   : byte IsWriteProtected()
  Description: lecture du switch de protection en criture
  Paramtres : Aucun
  Retour     : TRUE si protg en criture
               FLASE dans le cas contraire
-----------------------------------------------------------*/
byte IsWriteProtected(void);

/*-------------------------------------------------------------     
  Function   : void SocketInitialize(void)
  Description: Initialisation des ports utiliss par la SD Card
  Paramtres : Aucun
  Retour     : Aucun
----------------------------------------------------------------*/
void SocketInitialize(void);

/*-------------------------------------------------                 
  Function   : void ShutdownMedia(void)
  Description: Coupure alimentation de la carte SD
  Paramtres : Aucun
  Retour     : Aucun
----------------------------------------------------*/
void ShutdownMedia(void);

/*----------------------------------------
  Function   : SDC_Error MediaInitialize()
  Description: Initialisation de la carte SD et des variables associes
  Paramtre  : indicateurs sur la carte :
    isSDMMC : type MMC
    isWP    : protge en criture
  Retour     : nature de l'erreur
    sdcValid                Indicateur de carte valide
    sdcCardInitCommFailure  Aucune communication n'a pu s'tablir avec la carte
    sdcCardNotInitFailure   La carte n'est pas entre dans la phase "initialisation"
    sdcCardInitTimeout      "Timeout" pdt l'initialisation
    sdcCardTypeInvalid      Le type de carte n'a pas pu tre tabli
    sdcCardBadCmd           La carte n'a pas reconnu la commande
    sdcCardTimeout          "Timeout" pdt une lecture, criture ou effacement   
    sdcCardCRCError         Erreur CRC pdt une lecture 
    sdcCardDataRejected     Le "CRC" des donnes ne correspond pas
    sdcEraseTimedOut        "Timeout" pdt effacement
  Note : SDC_CS reste  "0" au retour en l'absence d'erreur
------------------------------------------*/
SDC_Error MediaInitialize(void);

/*------------------------------------------------------------------
  Function   : SDC_Error CSDRead(BYTE *buffer)
  Description: Lecture du registre CSD (Card Specific Data) :
    - transmission de la commande CMD9 SEND_CSD
    - rception des 128 bits du registre (16 octets). Infos extraites :
        - capacit de la carte
        - longueur (gblBlkLen) et nombre de blocs (gblNumBlks)
  Paramtres : buffer : destination des donnes lues 
  Retour     : nature de l'ventuelle erreur (voir SDC_Error)
-------------------------------------------------------------------*/
SDC_Error CSDRead(void);

/*----------------------------------------------------------------------
  Function   : SDC_Error SectorRead(uint32 sector_addr, Secteur* buffer)
  Description: la fonction lit 512 octets  partir de l'adresse "sector_addr"
               et range les octets dans "buffer"
  Paramtres :
   - sector_addr : adresse du secteur; chaque secteur comporte un bloc de 512 
                   octets
   - buffer : destination des donnes lues  (buffer de 512 octets)
  Retour     : voir SDC_Error
------------------------------------------------------------------------*/
SDC_Error SectorRead(uint32 sector_addr, Secteur* buffer);

/*---------------------------------------------------------------------
  Function   : SDC_Error SectorWrite(DWORD sector_addr, Secteur* buffer)
  Description: la fonction crit les 512 octets de "buffer"  partir de 
               l'adresse "sector_addr"
  Paramtres :
   - sector_addr : adresse du secteur; chaque secteur comporte un bloc de 512 octets
   - buffer : pointeur vers la source des donnes  crire (tableau de 512 octets)
  Retour     : voir SDC_Error
-----------------------------------------------------------------------*/
SDC_Error SectorWrite(uint32 sector_addr, Secteur* buffer);

/*--------------------------------------------------------          
  Function   : SDC_Error MediaInitialize()
  Description: Initialisation de la carte SD et des variables associes
  Paramtre  : indicateurs sur la carte :
    isSDMMC : type MMC
    isWP    : protge en criture
  Retour     : nature de l'erreur
    sdcValid                Indicateur de carte valide
    sdcCardInitCommFailure  Aucune communication n'a pu s'tablir avec la carte
    sdcCardNotInitFailure   La carte n'est pas entre dans la phase "initialisation"
    sdcCardInitTimeout      "Timeout" pdt l'initialisation
    sdcCardTypeInvalid      Le type de carte n'a pas pu tre tabli
    sdcCardBadCmd           La carte n'a pas reconnu la commande
    sdcCardTimeout          "Timeout" pdt une lecture, criture ou effacement   
    sdcCardCRCError         Erreur CRC pdt une lecture 
    sdcCardDataRejected     Le "CRC" des donnes ne correspond pas
    sdcEraseTimedOut        "Timeout" pdt effacement
  Note : SDC_CS reste  "0" au retour en l'absence d'erreur
-----------------------------------------------------------*/
SDC_Error MediaInitialize(void);

#endif // SD_Card_Defs
