     /********************************************************************
      ***                      Projet VAE                              ***
      ***               Fonctions lies  l'UART2                      ***
      ***                      dsPIC30F4011                            ***
      ***                                                              ***
      ***  Fichier "source" : UART2.c                                  ***
      ***  Auteur : CREMMEL Marcel                                     ***
      ***  Date   : 30/09/2007                                         ***
      ********************************************************************/

#include <p30f4011.h>
#include "Hardware.h"
#include "UART2.h"

/*------------------------------------------------------------------------------
                                  Variables                              
------------------------------------------------------------------------------*/

/* Buffers RX et TX pour l'UART2 (communication PC)
***************************************************/
#define RX2_BufSize 16          // Taille du Buffer_RX2
char Buffer_RX2[RX2_BufSize];   // Buffer circulaire de rception
char *ptrRX2_WRdata=Buffer_RX2; // Pointeur d'criture ds Buffer_RX2
char *ptrRX2_RDdata=Buffer_RX2; // Pointeur de lecture ds Buffer_RX2
#define TX2_BufSize 512         // Taille du Buffer_TX2
char Buffer_TX2[TX2_BufSize];   // Buffer circulaire de transmission
char *ptrTX2_WRdata=Buffer_TX2; // Pointeur d'criture ds Buffer_TX2
char *ptrTX2_RDdata=Buffer_TX2; // Pointeur de lecture ds Buffer_TX2

/*------------------------------------------------------------------------------
             Dclarations des fonctions en mmoire flash
------------------------------------------------------------------------------*/

/*---------------------------------------------------------------
  Function   : void InitUART2(void)
  Description: Initialisation de l'UART2
                 - 38400 bauds, 8 bits, pas de parit, 1 bit Stop
                 - interruption sur TX si buffer UART vide
                 - interruption sur RX  chaque caractre
------------------------------------------------------------------*/
void InitUART2(void)
{
 U2MODE=0x8000; // UART valid, 8 bits, pas de parit, 1 bit stop
 U2STA =0x8400; // Interruption TX si buffer UART vide
                // UART TX valid
                // Interruption RX  chaque caractre
 U2BRG =(FCY/(16*38400))-1; // 38400 bauds (erreur +0,16%)
}

/*-------------------------------------------------------------
  Function   : int ReadRXD2(char *c)
  Description: Lecture d'un octet dans le buffer de rception
                 - renvoi de 0 si aucun caractre reu
                 - renvoi de 1 si caractre reu :
                 - caractre renvoy dans "c"
                 - pointeur circulaire ptrRX2_RDdata incrment
----------------------------------------------------------------*/
int ReadRXD2(char *c)
{
 if (ptrRX2_RDdata==ptrRX2_WRdata) return(0); // Pas de caractre reu
 else
 {
  *c=*ptrRX2_RDdata++;// Affecter "c" avec le caractre reu puis incrmenter
                     // le pointeur ptrRX2_RDdata
  // Pointeur circulaire
  if (ptrRX2_RDdata==Buffer_RX2+RX2_BufSize) ptrRX2_RDdata=Buffer_RX2;
  return(1);
 }
}

/*--------------------------------------------------------------
  Function   : void WriteTXD2(char c)
  Description: Ecriture d'un octet dans le buffer d'mission
                 - attendre libration du buffer
                 - envoi du caractre reu
                 - pointeur circulaire ptrTX2_WRdata incrment
-----------------------------------------------------------------*/
void WriteTXD2(char c)
{
 int Nb_char;
 // Attendre libration d'une partie du buffer
 do
  {// Calcul du nb de caractres restant  transmettre
   Nb_char=ptrTX2_WRdata-ptrTX2_RDdata;
   if (Nb_char < 0) Nb_char=Nb_char+TX2_BufSize;
  } 
 while (Nb_char > (TX2_BufSize-8));
 // Caractre plac dans le buffer circulaire
 *ptrTX2_WRdata++=c; 
 if (ptrTX2_WRdata==Buffer_TX2+TX2_BufSize) ptrTX2_WRdata=Buffer_TX2;
 // Provoquer la 1 interruption si aucune transmission en cours
 // Le buffer de l'UART fait 4 octets (Nb_char=3)
 if ((U2STAbits.TRMT)&&(Nb_char>=3)) IFS1bits.U2TXIF=1; 
}

/*---------------------------------------------------------------
  Function   : void WriteIntTXD2(int i)
  Description: Ecriture d'un entier dans le buffer d'mission
  Paramtre  : Entier  transmettre en binaire
-----------------------------------------------------------------*/
void WriteIntTXD2(int i)
{
 WriteTXD2(i>>8); // Octet MSB d'abord
 WriteTXD2(i);    // Puis octet LSB
}

/*------------------------------------------------------------------
  Function   : void WriteTXD2_String(char *String)
  Description: Ecriture d'une chane de caractres dans le buffer TX
  Paramtre  : Pointeur de la chane de caractres
--------------------------------------------------------------------*/
void WriteTXD2_String(char *String)
{
 while (*String) // Attendre caractre final (0)
 {
  WriteTXD2(*String++); // Transmission du caractre suivant
 }
}

/*-------------------------------------------------------------------
  Function   : void WriteLnTXD2_String(char *String)
  Description: Ecriture d'une chane de caractres dans le buffer TX
               "Retour chariot" (CR+LF) ajout  la fin
  Paramtre  : Pointeur de la chane de caractres
---------------------------------------------------------------------*/
void WriteLnTXD2_String(char *String)
{
 WriteTXD2_String(String);    // Transmission de la chane
 WriteTXD2(CR); WriteTXD2(LF);// Transmission du "retour chariot"
}

/*---------------------------------------------
  Function   : void WriteLnTXD2(void
  Description: "Retour chariot" (CR+LF) sur TX2
-----------------------------------------------*/
void WriteLnTXD2(void)
{
 WriteTXD2(CR); WriteTXD2(LF);// Transmission du "retour chariot"
}

/*-------------------------------------------------------------
  Function   : void Conv_Bin8_String(char *String, byte x)
  Description: Conversion d'une valeur code 8 bits en binaire 
                 -> chane de caractres
  Paramtres : x : nombre binaire  convertir
               *String : pointeur vers la chane de destination
---------------------------------------------------------------*/
void Conv_Bin8_String(char *String, byte x)
{
 String[0]=x/100;                              // Centaines
 String[1]=(x-(100*String[0]))/10;             // Dizaines
 String[2]=x-(100*String[0])-10*String[1]+'0'; // Units en Ascii
 String[0]+='0'; // Conversion BCD -> Ascii des centaines
 String[1]+='0'; // Conversion BCD -> Ascii des dizaines
 String[3]=0;    // Terminateur de chane
}

/*-------------------------------------------------------------
  Function   : void Conv_Bin16_String(char *String, byte x)
  Description: Conversion d'une valeur code 16 bits en binaire 
                 -> chane de caractres
  Paramtres : x : nombre binaire  convertir
               *String : pointeur vers la chane de destination
---------------------------------------------------------------*/
void Conv_Bin16_String(char *String, uint x)
{ 
 uint Reste;
 String[0]=x/10000;    Reste=x-(10000*String[0]); // Dizaines de milliers
 String[1]=Reste/1000; Reste=Reste-(1000*String[1]); // Milliers
 String[2]=Reste/100;  Reste=Reste-(100*String[2]);  // Centaines
 String[3]=Reste/10;   Reste=Reste-(10*String[3]);   // Dizaines
 String[4]=Reste+'0'; // Units en Ascii
 String[0]+='0'; // Conversion BCD -> Ascii des centaines
 String[1]+='0'; // Conversion BCD -> Ascii des dizaines
 String[2]+='0'; // Conversion BCD -> Ascii des dizaines
 String[3]+='0'; // Conversion BCD -> Ascii des dizaines
 String[5]=0;    // Terminateur de chane
}

/*-------------------------------------------------------------
  Function   : void Conv_Bin32_String(char *String, byte x)
  Description: Conversion d'une valeur code 32 bits en binaire 
                 -> chane de caractres
  Paramtres : x : nombre binaire  convertir
               *String : pointeur vers la chane de destination
---------------------------------------------------------------*/
void Conv_Bin32_String(char *String, uint32 x)
{ 
 uint32 Reste;
 int i;
 String[0]=(uint32)x/1000000000;    
 Reste=x-(1000000000*String[0]); // 
 String[1]=Reste/100000000; Reste=Reste-(100000000*String[1]); // 
 String[2]=Reste/10000000;  Reste=Reste-(10000000*String[2]); // 
 String[3]=Reste/1000000;   Reste=Reste-(1000000*String[3]); // 
 String[4]=Reste/100000;    Reste=Reste-(100000*String[4]); // 
 String[5]=Reste/10000;     Reste=Reste-((uint32)10000*String[5]); // 
 String[6]=Reste/1000;      Reste=Reste-((uint32)1000*String[6]); // Milliers
 String[7]=Reste/100;       Reste=Reste-((uint32)100*String[7]);  // Centaines
 String[8]=Reste/10;        Reste=Reste-((uint32)10*String[8]);   // Dizaines
 String[9]=Reste+'0'; // Units en Ascii
 for (i=0; i<9; i++) String[i]+='0'; // Conversion BCD -> Ascii
 String[10]=0;    // Terminateur de chane
}

/*------------------------------------------------------------------- 
  Fonction    : void WTX2_Msg_Nb8bits(char *String, byte x)
  Description : Transmission d'un message Ascii suivi d'un  nombre 
                cod sur 8 bits       
  Paramtres  : *String : pointeur vers la chane de destination
                x : nombre binaire  convertir et transmettre 
---------------------------------------------------------------------*/
void WTX2_Msg_Nb8bits(char *String, byte x)
{
 char s[4];
 WriteTXD2_String(String); // Transmission de la chane
 Conv_Bin8_String(s,x);    // Conversion BN -> Ascii
 WriteTXD2_String(s);      // Transmission de la chane
 WriteTXD2(CR); WriteTXD2(LF);// Transmission du "retour chariot"
}

/*------------------------------------------------------------------- 
  Fonction    : void WTX2_Msg_Nb16bits(char *String, byte x)
  Description : Transmission d'un message Ascii suivi d'un  nombre 
                cod sur 16 bits       
  Paramtres  : *String : pointeur vers la chane de destination
                x : nombre binaire  convertir et transmettre 
---------------------------------------------------------------------*/
void WTX2_Msg_Nb16bits(char *String,uint x)
{
 char s[8];
 WriteTXD2_String(String); // Transmission de la chane
 Conv_Bin16_String(s,x);   // Conversion BN -> Ascii
 WriteTXD2_String(s);      // Transmission de la chane
 WriteTXD2(CR); WriteTXD2(LF);// Transmission du "retour chariot"
}

/*------------------------------------------------------------------- 
  Fonction    : void WTX2_Msg_Nb32bits(char *String, byte x)
  Description : Transmission d'un message Ascii suivi d'un  nombre 
                cod sur 32 bits
  Paramtres  : *String : pointeur vers la chane de destination
                x : nombre binaire  convertir et transmettre 
---------------------------------------------------------------------*/
void WTX2_Msg_Nb32bits(char *String,uint32 x)
{
 char s[11];
 WriteTXD2_String(String); // Transmission de la chane
 Conv_Bin32_String(s,x);   // Conversion BN -> Ascii
 WriteTXD2_String(s);      // Transmission de la chane
 WriteTXD2(CR); WriteTXD2(LF);// Transmission du "retour chariot"
}

/*----------------------------------------------------------------    
  Function    : void _ISR _U2TXInterrupt (void)
  Description : Programme d'interruption dclench quand le buffer
                de transmission de l'UART (4 octets) est vide
-----------------------------------------------------------------*/
void _ISR _U2TXInterrupt (void)
{
 if (ptrTX2_RDdata!=ptrTX2_WRdata) // Y a-t-il un caractre  transmettre ?
 {
  do
  {
   U2TXREG=*ptrTX2_RDdata++;//Transm. du caractre et incrmentation du pointeur
   // Buffer circulaire
   if (ptrTX2_RDdata==Buffer_TX2+TX2_BufSize) ptrTX2_RDdata=Buffer_TX2;
  }
  while ((ptrTX2_RDdata!=ptrTX2_WRdata)&&(!U2STAbits.UTXBF));
 }
 IFS1bits.U2TXIF=0; // Acquitement de l'interruption
}

/*---------------------------------------------------------------     
  Function    : void _ISR _U2RXInterrupt (void)
  Description : Programme d'interruption dclench  la rception
                d'un caractre
----------------------------------------------------------------*/
void _ISR _U2RXInterrupt (void)
{
 char c;
 IFS1bits.U2RXIF=0;      // Clear interrupt flag
 while (U2STAbits.URXDA) // Boucle pour lire ts les car. ds le buf. de l'UART
 {
  c=U2RXREG;
  *ptrRX2_WRdata++=c; // Lecture du caractre suivant et incrmentation
                     // du pointeur circulaire de lecture
  if (ptrRX2_WRdata==Buffer_RX2+RX2_BufSize) ptrRX2_WRdata=Buffer_RX2;
 }
}

/*---------------------------------------------------------------     
  Function    : int write(int handle, char *buffer, uint len)
  Description : Fonction appeles par les fonctions de sortie de 
                la librairie stdio
  Paramtres  : *buffer : prochain caractre  "sortir"
  Retour      : 0 si erreur, sinon "len"
----------------------------------------------------------------*/
int write(int handle, char *buffer, uint len)
{
 WriteTXD2(*(char*)buffer);
 return len;
}
 
